@@ -8,7 +8,7 @@ import logbook
 from .util import dirch, checked_output, confirm, tmp_virtualenv, tmp_checkout
 from .exc import ReleaseError
 from .version import NormalizedVersion, find_version
-from .git import prepare_commit
+from .git import prepare_commit, diff_tree
 
 log = logbook.Logger('unleash')
 
@@ -18,14 +18,14 @@ def action_create_release(args, repo):
     if not refname in repo.refs:
         raise ValueError('Could not find %s' % refname)
 
-    commit_id = repo.refs[refname]
-    tree = repo.object_store[repo.object_store[commit_id].tree]
+    commit = repo[refname]
+    tree = repo[commit.tree]
     log.debug('tree found: %s' % tree.id)
 
     # retrieve version from setup.py
     try:
         setuppy_mode, setuppy_id = tree['setup.py']
-        setuppy = repo.object_store[setuppy_id]
+        setuppy = repo[setuppy_id]
     except KeyError:
         raise KeyError('setup.py not found in repository root')
 
@@ -62,10 +62,10 @@ def action_create_release(args, repo):
 
     # prepare the commits
     release_commit, release_tree, release_objs = prepare_commit(
-        repo, commit_id, release_version, args.author, msg_release
+        repo, commit.id, release_version, args.author, msg_release
     )
     dev_commit, dev_tree, dev_objs = prepare_commit(
-        repo, commit_id, dev_version, args.author, msg_dev
+        repo, commit.id, dev_version, args.author, msg_dev
     )
 
     objects_to_add = []
@@ -118,6 +118,26 @@ def action_create_release(args, repo):
     log.info('Setting %s to %s' % (tag_refname, release_commit.id))
     repo.refs[tag_refname] = release_commit.id
 
+    if not repo.bare:
+        repo_prefix = 'ref: '
+
+        # if HEAD points to the updated branch, update the index
+        if not repo.refs.read_ref('HEAD') == repo_prefix + refname:
+            log.warning('HEAD does not point to %s, but %s. Will not update '
+                        'index.' % (refname, repo.refs.read_ref('HEAD')))
+        elif not repo.has_index():
+            log.warning('Repo has no index. Why?')
+        else:
+            index = repo.open_index()
+            if [c for c in index.changes_from_tree(
+                repo.object_store, repo[dev_commit.parents[0]].tree)
+            ]:
+                # the index has wasn't clean before
+                log.warning('Index was not clean, not checking out new copy.')
+            else:
+                # FIXME: missing - check if working copy has changes
+                log.warning('(not implemented): Should update index.')
+
 
 def action_publish(args, repo):
     import verlib
@@ -145,10 +165,10 @@ def action_publish(args, repo):
     if not args.version:
         raise ValueError('No version given and no version tag found.')
 
-    commit_id = repo.refs[prefix + args.version]
-    log.info('Checking out %s (%s)...' % (args.version, commit_id))
+    commit = repo[prefix + args.version]
+    log.info('Checking out %s (%s)...' % (args.version, commit.id))
 
-    with tmp_checkout(repo, commit_id) as src, tmp_virtualenv() as venv:
+    with tmp_checkout(repo, commit.id) as src, tmp_virtualenv() as venv:
         log.info('Uploading to PyPI...')
         with dirch(src):
             python = os.path.join(venv, 'bin', 'python')
@@ -156,7 +176,7 @@ def action_publish(args, repo):
 
             cmd = [python, 'setup.py', 'sdist', 'upload']
             if args.sign:
-                cmd.append('-s', '-i', args.sign)
+                cmd.extend(['-s', '-i', args.sign])
             checked_output(cmd)
 
     with dirch(repo.path):
